/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Roman Khler <roman.koehler@aizo.com>
 *  Complete rewrite by: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

var LOGFILE_NAME = 'timed-events.log';
var LOG = new Logger(LOGFILE_NAME);
var WEEK = ['MO','TU','WE','TH','FR','SA','SU'];

function pad(c) {
    var i = parseInt(c, 10);
    return (i < 10 ? '0' + i : c.toString());
}

function getDayIdx(d) {
    var i = 0;
    for (; i < 7; ++i) {
        if (d == WEEK[i]) {
            return i;
        }
    }
    return -1;
}

/**
 * A function that converts stored timed events from the old to the new version
 */
function convertOldEvents() {
    LOG.logln('Converting events from old timed events app');
    Property.load();
    var events = Property.getNode('events').getChildren();
    var i = 0;
    for (; i < events.length; ++i) {
        var ndOldParams = events[i].getChild('params');
        var oldParams = JSON.parse(ndOldParams.getValue());
        var oldRecurrence = JSON.parse(decodeURIComponent(params.recurrence));
        var oldActions = JSON.parse(decodeURIComponent(params.actions));

        //
        // validity checks
        //

        if (!oldRecurrence || !oldActions || oldActions.length == 0) {
            // delete invalid events
            LOG.logln('Deleting event '+ events[i].name +': missing recurrence or actions');
            deleteOldEvent(events[i]);
            continue;
        }
        if (! oldRecurrence.triggerTime
            || oldRecurrence.recurrence == 'irregular'
               && (!oldRecurrence.dates
                   || oldRecurrence.dates.length == 0)
            || oldRecurrence.recurrence == 'regular_wk'
               && (!oldParam.weekdays
                   || oldParam.weekdays.length == 0)
        ) {
            LOG.logln('Deleting event '+ events[i].name +': missing time, dates or week days');
            deleteOldEvent(events[i]);
            continue;
        }

        //
        // Conversions
        //

        // new params object
        var params = {
            name: oldParams.name
        };

        // disabled event
        if (oldParams.Disabled === true) {
            params.disabled = true;
        }

        // Convert time from UTC to local time
        // (assuming correctly set timezone on the dSS)
        var utc = new Date();
        var time = oldRecurrence.triggerTime.split(':');
        utc.setUTCHours(parseInt(time[0], 10));
        utc.setUTCMinutes(parseInt(time[1], 10));
        utc.setUTCSeconds(parseInt(time[2], 10));
        params.time = // utc.format('H:i:s');
                    pad(utc.getHours())
                    + ':' + pad(utc.getMinutes())
                    + ':' + pad(utc.getSeconds());

        // recurrence period
        if (oldRecurrence.recurrence == 'regular_wk') {
            params.recurring = true;
        }

        if (params.recurring) {
            // weekdays
            params.recurrence = [];
            var j = 0;
            for (; j < oldRecurrence.weekdays.length; ++j) {
                var idx = getDayIdx(oldRecurrence.weekdays[j]);
                if (idx !== -1) {
                    params.recurrence.push(idx);
                } else {
                    LOG.logln('Skipping invalid weekday: ' + oldRecurrence.weekdays[j]);
                    continue;
                }
            }
            if (params.recurrence.length == 0) {
                LOG.logln('Deleting event '+ events[i].name +': no valid weekdays');
                deleteOldEvent(events[i]);
                continue;
            }

        } else {
            // reverse the dates from 13-01-2001 to 2001-01-13
            var dates = oldRecurrence.dates;
            var d;
            params.recurrence = [];
            for (d = 0; d < dates.length; ++d) {
                var date = dates[d].split('-');
                if (date.length != 4
                    || parseInt(date[1], 10) > 12) {
                    LOG.logln('Skipping invalid date');
                    continue;
                }
                params.recurrence.push(date[2] +'-'+ date[1] +'-'+ date[0]);
            }
            if (params.recurrence.length == 0) {
                LOG.logln('Deleting event '+ events[i].name +': no valid dates');
                deleteOldEvent(events[i]);
                continue;
            }
        }

        // store converted event
        events[i].removeChild(ndOldParams);
        storeParams(params);
    }
}

/**
 * Execute actions delayed one by one
 * @param actions {Array} Actions to execute (in the order of the array)
 * @param time {Number} Miliseconds to wait between actions
 */
function executeActions(actions, time) {
    var action = actions.shift();

    var target = null;
    if (action.group != null) {
        LOG.logln('target zone ' + action.zone + ' group ' + action.group);
        target = getDevices().byZone(action.zone).byGroup(action.group);

    } else if (action.device != null) {
        LOG.logln('target device ' + action.device);
        target = getDevices().byDSID(action.device);
    }

    if (target) {
        if (action.scene) {
            LOG.logln('Exec: calling scene ' + action.scene);
            target.callScene(parseInt(action.scene, 10));
        }
        else if (action.blink) {
            LOG.logln('Exec: blinking');
            target.blink();
        }
        else if (action.outputValue) {
            LOG.logln('Exec: setting ouput ' + action.outputValue);
            target.setValue(parseInt(action.outputValue, 10));
        }
    } else {
        LOG.logln('Warning: empty target');
    }

    if (actions.length > 0) {
        LOG.logln('Scheduling next '+ actions.length +' params in '+ time +'ms');
        var o = function () { executeActions(actions, time); };
        setTimeout(time, o);
    }
} // executeActions

/**
 * Parses the recurrence object and generates an array of strings, each
 * containing a valid recurrence rule.
 * The dSS only accepts iCal dates if one provides both rrule and start time
 * @param params {Object} The event's parameters object
 * @return {Array} An array of rrule/starting time pair objects
 */
function generateIcalRules(params) {
    var rules = [];
    var time = params.time.split(':');
    var d = new Date();
    var i;
    var utc = true; // default to compatibility mode with old dSS versions
    d.setHours(parseInt(time[0], 10));
    d.setMinutes(parseInt(time[1], 10));
    d.setSeconds(parseInt(time[2], 10));

    var timestring = '';
    if (utc) {
        // run in compatibility mode
        timestring = pad(d.getUTCHours())
            + pad(d.getUTCMinutes())
            + pad(d.getUTCSeconds())
            + 'Z';
    } else {
        // use local time
        timestring = pad(d.getHours())
            + pad(d.getMinutes())
            + pad(d.getSeconds());
    }

    var rrule = null;
    if (params.recurring) {
        var iCalStartTime =
            d.getFullYear()
            + pad(d.getMonth() + 1) // goes from 0..11
            + pad(d.getDate())
            + 'T'
            + timestring;

        var weekdays = [];
        for (i = 0; i < params.recurrence.length; ++i) {
            weekdays.push(WEEK[params.recurrence[i]]);
        }
        rrule = 'FREQ=WEEKLY;BYDAY=' + weekdays;
        LOG.logln('Debug: rrule is `'+ rrule +'\'');

        rules.push({
            iCalRRule: rrule,
            iCalStartTTime: iCalStartTime
        });

    } else {
        rrule = 'FREQ=DAILY;COUNT=1';
        for (i = 0; i < params.recurrence.length; i++) {
            var iCalStartTime = params.recurrence[i] + 'T' + timestring;
            rules.push({
                iCalRRule: rrule,
                iCalStartTTime: iCalStartTime
            });
        }
    }

    return rules;
}

/**
 * Check if all timed-events are in the queue, else requeue them
 */
function requeueEvents() {
    var events = Property.getNode('events').getChildren();
    var i = 0;
    var j, k;

    for (; i < events.length; ++i) {
        var queue = events[i].getChild('queueIDs').getChildren();
        var params = null;

        for (j = 0; j < queue.length; ++j) {
            var propNode = Property.getNode(
                '/system/EventInterpreter/ScheduledEvents/' + queue[j].getValue()
            );
            if (!propNode) {
                // re-raise the event and store the queue id
                if (!params) {
                    // cache params
                    params = loadParams(events[i]);
                }
                var ical_rules = generateIcalRules(params);

                LOG.logln('queueing events for timer ' + params.name);

                for (k = 0; k < ical_rules.length; ++k) {
                    var timerEvent = new TimedICalEvent(
                        'timed-events',
                        ical_rules[k].iCalStartTTime,
                        ical_rules[k].iCalRRule,
                        {
                            timername: params.name,
                            action_type: 'execute'
                        }
                    );
                    Property.setProperty('events/' + params.name + '/queueIDs/' + i, timerEvent.raise());
                }
            }
        }
    }
}

/**
 * Fetches the parameters of this event from the property tree.
 * @param eventNode {Object} Property tree node of the event
 * @return {Object} Object containing the read out values of the event
 */
function loadParams(eventNode) {
    LOG.logln('Loading params for node: ' + eventNode.name);
    var name = eventNode.getChild('name');
    if (!name) {
        LOG.logln('Error: empty or invalid node');
        return;
    }

    var params = {
        name: name.getValue(),
        time: eventNode.getChild('time').getValue(),
        actions: [],
        recurrence: []
    };
    var ndDisabled = eventNode.getChild('disabled');
    params.disabled = (ndDisabled ? !! ndDisabled.getValue() : false);

    LOG.logln('Loading actions');
    var ndActions = eventNode.getChild('actions');
    var loadValue = function(action, ndAction, name) {
        var c = ndAction.getChild(name);
        if (c) {
            action[name] = c.getValue();
            return true;
        }
        return false;
    };
    if (ndActions) {
        var actions = ndActions.getChildren();
        var a = 0;
        for (; a < actions.length; ++a) {
            var curAction = {};
            if (loadValue(curAction, actions[a], 'device')) {}
            else if (loadValue(curAction, actions[a], 'group') && loadValue(curAction, actions[a], 'zone')) {}
            else {
                LOG.logln('Error: event '+ params.name +' has an action with no target');
                continue;
            }
            if (loadValue(curAction, actions[a], 'blink')) {}
            else if (loadValue(curAction, actions[a], 'scene')) {}
            else if (loadValue(curAction, actions[a], 'outputValue')) {}
            else {
                LOG.logln('Error: event '+ params.name +' has an action with invalid action to take');
                continue;
            }
            params.actions.push(curAction);
            LOG.logln('action: '+ JSON.stringify(curAction));
        }
    } else {
        LOG.logln('Error: event '+ params.name +' has no actions set');
    }

    LOG.logln('Loading recurrences');
    var ndRecurring = eventNode.getChild('recurring');
    params.recurring = (ndRecurring ? !!ndRecurring.getValue() : false);

    var ndRecurrence = eventNode.getChild('recurrence');
    if (ndRecurrence) {
        var recurrences = ndRecurrence.getChildren();
        if (params.recurring) {
            // weekdays
            var w = 0;
            for (; w < recurrences.length; ++w) {
                params.recurrence.push(recurrences[w].name);
            }
        } else {
            // dates
            var d = 0;
            for (; d < recurrences.length; ++d) {
                params.recurrence.push(recurrences[d].getValue());
            }
        }
        LOG.logln('recurrences: '+ JSON.stringify(params.recurrence));
    } else {
        LOG.logln('Error: event '+ params.name +' has no recurrence set');
    }
    LOG.logln('params loaded');
    return params;
}

/**
 * Updates a node if it exists or creates a new archived property node otherwise
 * @param path {String} Relative path
 * @param value {String|Boolean|Integer} Value to set
 */
function updateOrCreateNode(path, value) {
    var nd = Property.getNode(path);
    if (nd) {
        nd.setValue(value);
    } else {
        createNode(path, value);
    }
}

/**
 * Creates a new archived property node with given value
 * @param path {String} Relative path
 * @param value {String|Boolean|Integer} Value to set
 * @return {Object} The newly created node
 */
function createNode(path, value) {
    var nd = new Property(path);
    if (value !== undefined)
        nd.setValue(value);
    nd.setFlag('ARCHIVE', true);
    return nd;
}

/**
 * Stores _VALID_ parameters into the property tree.
 * Use checkParams(params) to validate them first!
 * @param params {Object} Object to store
 */
function storeParams(params) {
    var eventPath = 'events/' + params.name;
    var actionPath = eventPath + '/actions';
    var i, c;

    var ndEvent = Property.getNode(eventPath) || createNode(eventPath);

    updateOrCreateNode(eventPath + '/name', params.name);
    updateOrCreateNode(eventPath + '/time', params.time);
    updateOrCreateNode(eventPath + '/disabled', !!params.disabled);

    // actions
    var ndActions = ndEvent.getChild('actions');
    if (!ndActions) {
        ndActions = createNode(actionPath);
    } else {
        // clean out actions, as indices might not match up
        for (c in ndActions.getChildren()) {
            ndActions.removeChild(c);
        }
    }
    // populate actions
    for (i = 0; i < params.actions.length; ++i) {
        var p = actionPath + '/' + i;
        createNode(p);

        if (params.actions[i].zone !== undefined)
            createNode(p + '/zone', params.actions[i].zone);
        if (params.actions[i].group !== undefined)
            createNode(p + '/group', params.actions[i].group);
        if (params.actions[i].device)
            createNode(p + '/device', params.actions[i].device);
        if (params.actions[i].blink)
            createNode(p + '/blink', true);
        if (params.actions[i].scene !== undefined)
            createNode(p + '/scene', params.actions[i].scene);
        if (params.actions[i].outputValue !== undefined)
            createNode(p + '/outputValue', params.actions[i].outputValue);
    }

    // recurrence
    updateOrCreateNode(eventPath + '/recurring', params.recurring);

    var ndRecurrence = ndEvent.getChild('recurrence')
                    || createNode(eventPath + '/recurrence');
    if (params.recurring) {
        // weed out existing weekdays
        for (c in ndRecurrence.getChildren()) {
            ndRecurrence.removeChild(c);
        }
        for (i = 0; i < params.recurrence.length; ++i) {
            var idx = params.recurrence[i];
            createNode(eventPath + '/recurrence/' + idx, WEEK[idx]);
        }

    } else {
        // weed out dates
        for (c in ndRecurrence.getChildren()) {
            ndRecurrence.removeChild(c);
        }
        for (i = 0; i < params.recurrence.length; ++i) {
            createNode(eventPath + '/recurrence/' + i, params.recurrence[i]);
        }
    }

    Property.store();
} // storeParams

/**
 * Checks if parameters conform to what is expected
 * @param eventParams {Object} Object to check
 * @return {Object} Conform object or null on error
 */
function checkParams(eventParams) {
    if (!eventParams.name
        || eventParams.name.length === 0
        || eventParams.name.indexOf('/') !== -1
    ) {
        LOG.logln('Error: event has no or invalid name');
        return null;
    }

    if (!eventParams.actions || eventParams.actions.length === 0) {
        LOG.logln('Error: event has no actions');
        return null;
    }

    if (!eventParams.recurrence || eventParams.recurrence.length === 0) {
        LOG.logln('Error: event has no recurrence');
        return null;
    }

    if (!eventParams.time) {
        LOG.logln('Error: event has no time set');
        return null;
    }

    var actions = [];
    var i = 0;
    for (; i < eventParams.actions.length; ++i) {
        var a = eventParams.actions[i];
        if (!a.device && isNaN(a.group)) { // isNan(undefined) === true
            LOG.logln('Error: no or invalid group or device defined as target on action ' + i);
            return null;
        }
        if (a.blink === undefined && a.scene === undefined && a.outputValue === undefined) {
            LOG.logln('Error: no action defined at index ' + i);
            return null;
        }
        if (
            a.scene !== undefined && isNaN(a.scene)
            || a.group !== undefined && isNaN(a.group)
            || a.outputValue !== undefined && (
                isNaN(a.outputValue) || a.outputValue < 0 || a.outputValue > 255
            )
        ) {
            LOG.logln('Error: invalid action defined at index ' + i);
            return null;
        }

        var action = {};
        if (a.zone !== undefined)
            action.zone = parseInt(a.zone, 10);
        if (a.group !== undefined)
            action.group = parseInt(a.group, 10);
        if (a.device !== undefined)
            action.device = a.device;
        if (a.blink)
            action.blink = true;
        if (a.scene !== undefined)
            action.scene = parseInt(a.scene, 10);
        if (a.outputValue !== undefined)
            action.outputValue = parseInt(a.outputValue, 10);
        actions.push(action);
    }

    var recurring = !!eventParams.recurring;
    var recurrence = [];
    if (recurring) {
        for (i = 0; i < eventParams.recurrence.length; ++i) {
            var d = eventParams.recurrence[i];
            if (isNaN(d) || d < 0 || d > 6) {
                LOG.logln('Warning: wrong weekday index `'+ d +'\', skipping');
                continue;
            }
            recurrence.push(d);
        }
    } else {
        for (i = 0; i < eventParams.recurrence.length; ++i) {
            var d = eventParams.recurrence[i];
            var date = d.split('-');
            if (date.length !== 3 || isNaN(date[0]) || isNaN(date[1]) || isNaN(date[2])
                || date[0].length < 4 || date[1] == 0 || date[1] > 12
                || date[2] == 0 || date[2] > 31
            ) {
                LOG.logln('Warning: invalid date `'+ d +'\'');
                continue;
            }
            recurrence.push(d);
        }
    }
    if (recurrence.length == 0) {
        LOG.logln('Error: no valid recurrence');
        return null;
    }
    var t = eventParams.time.split(':');
    var inv = (t.length !== 3);
    for (i = 0; !inv && i < 3; ++i) {
        if (!isNaN(t[i])) {
            t[i] = parseInt(t[i], 10);
            inv = inv || t[i] < 0 || t[i] > 59;
        } else {
            inv = true;
        }
    }
    inv = inv || (t[0] > 23);
    if (inv) {
        LOG.logln('Error: invalid time format');
        return null;
    }
    var time = pad(t[0]) + ':' + pad(t[1]) + ':' + pad(t[2]);

    // create the valid params object
    var params = {
        name: eventParams.name,
        actions: actions,
        disabled: !!eventParams.disabled,
        recurring: recurring,
        recurrence: recurrence,
        time: time
    };

    return params;
} // checkParams

/**
 * Add a new event
 * @param params {Object} The new event is created from these parameters
 */
function addEvent(eventParams) {
    // check eventParams and set params for serialization
    var params = checkParams(eventParams);
    if (!params)
        return;
    storeParams(params);

    var ical_rules = generateIcalRules(params);
    var i = 0;
    for (; i < ical_rules.length; i++) {
        LOG.logln('Queuing events for ' + params.name);

        var timerEvent = new TimedICalEvent(
            'timed-events',
            ical_rules[i].iCalStartTTime,
            ical_rules[i].iCalRRule,
            {
                timername: params.name,
                action_type: 'execute'
            }
        );

        Property.setProperty('events/' + params.name + '/queueIDs/' + i, timerEvent.raise());
        Property.setFlag('events/' + params.name + '/queueIDs/' + i, 'ARCHIVE', true);
    }

    // set archive flags
    Property.setFlag('events/' + params.name + '/queueIDs', 'ARCHIVE', true);
    Property.setFlag('events/' + params.name, 'ARCHIVE', true);
    Property.setFlag('events', 'ARCHIVE', true);
    Property.store();
}

function executeEvent(eventNode, params) {
    LOG.logln('Processing event ' + params.name);

    LOG.logln('- '+ params.actions.length +' actions');
    if (params.disabled === true) {
        disabled = true;
        LOG.logln('Event '+ eventName +' is disabled');
    } else {
        executeActions(params.actions, 500);
    }

    // Delete irregular event if it's its last occurrence
    if (!params.recurring) {
        if (eventNode.getChild('queueIDs').getChildren().length == 1) {
            deleteEvent(eventNode, params);

            // notify UI about changes
            var updateEvent = new Event('timed-events.notifyUi', params.name);
            updateEvent.raise();
        }
    }
}

function deleteOldEvent(eventNode) {
    deleteEvent(eventNode, { name: eventNode.name });
}

function deleteEvent(eventNode, params) {
    LOG.logln('Deleting event '+ params.name);

    var queue = eventNode.getChild('queueIDs');
    if (queue) {
        var qchildren = queue.getChildren();
        var queueNode = Property.getNode('/system/EventInterpreter/ScheduledEvents');
        var i = 0;
        for (; i < qchildren.length; ++i) {
            queueNode.removeChild(qchildren[i].getValue());
        }
    }

    eventNode.getParent().removeChild(eventNode);
    Property.store();
}

/**
 * Enable or disable a timed event
 * @param enable {Boolean} True to enable, false to disable
 */
function enableEvent(eventNode, params, enable) {
    LOG.logln((enable ? 'Enabling' : 'Disabling') + ' event ' + params.name);

    params.disabled = !enable;
    updateOrCreateNode('event/' + params.name + '/disabled', !enable);
    Property.store();
}

function main() {
    LOG.logln('timed-events main call with parameters: '
                + raisedEvent.parameter.toString());
    if (raisedEvent.parameter.action_type == 'add') {
        // Add a new event
        if (raisedEvent.parameter.params === undefined) {
            LOG.logln('Error: missing event parameters');
            return;
        }
        LOG.logln('Add event with parmeters: '+ raisedEvent.parameter.params);
        var params = JSON.parse(raisedEvent.parameter.params);
        addEvent(params);

    } else if (raisedEvent.name == 'running') {
        // Prepare app
        Property.load();
        LOG.logln('timed-events running');

        var ndVersion = Property.getNode('version');
        if (ndVersion) {
            var version = ndVersion.getValue().split('.');
            if (version.length > 1 && version[0] == '0' && parseInt(version[1], 10) < 9) {
                convertOldEvents();
            }
        }
        var events = new Property('events');
        events.setFlag('ARCHIVE', true);
        //set new version number
        Property.setProperty('version', '0.9.0');
        Property.setFlag('version', 'ARCHIVE', true);
        Property.store();

    } else {
        // Operations that involve a specific event's parameters
        Property.load();
        var ndEvent = Property.getNode('events/' + raisedEvent.parameter.event_name);
        if (ndEvent) {
            var params = loadParams(ndEvent);

            if (raisedEvent.parameter.action_type == 'enable') {
                enableEvent(ndEvent, params, true);
            }
            else if (raisedEvent.parameter.action_type == 'disable') {
                enableEvent(ndEvent, params, false);
            }
            else if (raisedEvent.parameter.action_type == 'delete') {
                deleteEvent(ndEvent, params);
            }
            else if (raisedEvent.parameter.action_type == 'execute') {
                executeEvent(ndEvent, params);
            }
        } else {
            LOG.logln('Error: '+ raisedEvent.parameter.event_name +' doesn\'t exist');
        }
    }
} // main

main();

