/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.namespace('DSS.component');

/**
 * Base class for DSS template components
 */
Ext.define('DSS.component.Base', {
    extend: 'Ext.container.Container',
    layout: 'anchor'
});

/** Possible target types */
Ext.define('DSS.component.TargetType', {
    statics: {
        ZONE: 'zone',
        DEVICE: 'dev'
    }
});
/** Possible action types */
Ext.define('DSS.component.ActionType', {
    statics: {
        ACTIVITY: 'activity',
        BLINK: 'blink',
        OUTPUT: 'output'
    }
});

/**
 * A selector that shows every day of the week as button
 */
Ext.define('DSS.component.WeekdaySelection', {
    alias: ['widget.dssWeekdaySelection'],
    extend: 'Ext.container.ButtonGroup',
    constructor: function(config) {
        this.initConfig(config);
        this.callParent(arguments);
    },
    config: {
        width: 235
    },
    defaults: {
        allowEmpty: false,
        enableToggle: true
    },
    days: ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun'],
    items: [
        { itemId: 'mon', text: _('Mon') },
        { itemId: 'tue', text: _('Tue') },
        { itemId: 'wed', text: _('Wed') },
        { itemId: 'thu', text: _('Thu') },
        { itemId: 'fri', text: _('Fri') },
        { itemId: 'sat', text: _('Sat') },
        { itemId: 'sun', text: _('Sun') },
    ],

    /**
     * Reset all values to default (unset) or specified values
     * Note: this does not trigger the toggle event
     * @param values (optional) values to reset to - default if unset
     */
    resetComponent: function(values) {
        var i = 0;
        if (values === undefined) {
            for (; i < this.days.length; ++i) {
                this.getComponent(this.days[i]).toggle(false, true);
            }
        } else {
            for (; i < this.days.length; ++i) {
                this.getComponent(this.days[i]).toggle(
                    Ext.Array.contains(values, i),
                    true
                );
            }
        }
    },

    isValid: function() {
        return this.allowEmpty || this.getValues().length > 0;
    },

    /**
     * Returns the currently pressed days
     * @return {[int]} Array of integers: 0=monday, ..., 6=sunday
     */
    getValues: function() {
        var pressed = [];
        var i = 0;
        for (; i < this.days.length; ++i) {
            if (this.getComponent(this.days[i]).pressed)
                pressed.push(i);
        }
        return pressed;
    }
});

/**
 * Recurrence Selection Panel
 */
Ext.define('DSS.component.RecurrenceSelection', {
    alias: ['widget.dssRecurrenceSelection'],
    extend: 'DSS.component.Base',
    uses: ['DSS.component.WeekdaySelection'],

    /** @private */
    recurring: true,
    weekdaySelection: null,
    dateField: null,

    initComponent: function() {
        var me = this;
        me.weekdaySelection = Ext.create('DSS.component.WeekdaySelection');
        me.dateField = Ext.create('Ext.form.field.Date', {
            allowBlank: false,
            hidden: true,
            name: 'onetimeDate',
            value: new Date()
        });
        Ext.apply(me, {
            //layout: 'anchor',
            items:
            [
                {
                    xtype: 'fieldcontainer',
                    itemId: 'time',
                    fieldLabel: _('Time'),
                    layout: 'column',
                    defaultType: 'numberfield',
                    defaults: {
                        allowBlank: false,
                        allowDecimals: false,
                        width: 50
                    },
                    items: [{
                        itemId: 'hours',
                        value: 12,
                        minValue: 0,
                        maxValue: 23
                    }, {
                        itemId: 'minutes',
                        value: 0,
                        minValue: 0,
                        maxValue: 59
                    }, {
                        itemId: 'seconds',
                        value: 0,
                        minValue: 0,
                        maxValue: 59
                    }]
                }, {
                    itemId: 'days',
                    xtype: 'fieldcontainer',
                    fieldLabel: _('Recurrence'),
                    layout: 'column',
                    items: [
                        {
                            xtype: 'cycle',
                            itemId: 'cyc-recurrence',
                            listeners: {
                                change: function(cycle, item, options) {
                                     me.setRecurring(item.itemId == 'chk-weekly');
                                }
                            },
                            menu: {
                                items: [{
                                    checked: true,
                                    itemId: 'chk-weekly',
                                    text: _('Weekly event')
                                }, {
                                    text: _('Specific dates')
                                }]
                            },
                            showText: true,
                            width: 150
                        }, {
                            xtype: 'container',
                            autoDestroy: false,
                            columnWidth: 0.5,
                            itemId: 'recurrenceSelection',
                            items: [
                                me.weekdaySelection,
                                me.dateField
                            ],
                            padding: '0 0 0 20'
                        }
                    ]
                }
            ]
        });
        me.callParent(arguments);
    },

    /**
     * Switch to change between displaying of recurring or one time event
     * @param recurring {Boolean} true: set recurring event, false: one time event
     */
    setRecurring: function(recurring) {
        var me = this;
        if (me.recurring == recurring)
            return;

        var cycle = me.getComponent('days').getComponent('cyc-recurrence');
        if (recurring) {
            me.dateField.hide();
            me.weekdaySelection.show();
            cycle.setActiveItem(cycle.menu.items.getAt(0), true);
        } else {
            me.weekdaySelection.hide();
            me.dateField.show();
            cycle.setActiveItem(cycle.menu.items.getAt(1), true);
        }
        me.recurring = recurring;
    },

    /**
     * Reset all values to default (unset) or specified values
     * @param values (optional) values to reset to - default if unset
     */
    resetComponent: function(values) {
        var me = this;
        var timebox = me.getComponent('time');
        if (values === undefined) {
            me.setRecurring(true);
            timebox.getComponent('hours').setValue(12);
            timebox.getComponent('minutes').setValue(0);
            timebox.getComponent('seconds').setValue(0);
            var cycle = me.getComponent('days').getComponent('cyc-recurrence');
            cycle.setActiveItem(cycle.menu.items.first());
            me.weekdaySelection.resetComponent();
            me.dateField.setValue(new Date());
        } else {
            me.setRecurring(!!values.recurring);
            timebox.getComponent('hours').setValue(values.hours);
            timebox.getComponent('minutes').setValue(values.minutes);
            timebox.getComponent('minutes').setValue(values.seconds);
            if (values.recurring) {
                me.weekdaySelection.resetComponent(values.recurrence);
                me.dateField.setValue(new Date());
            } else {
                me.weekdaySelection.resetComponent();
                // TODO: add support for multiple dates
                me.dateField.setValue(values.recurrence[0]);
            }
        }
    },

    isValid: function() {
        var timebox = this.getComponent('time');
        return (
            (this.recurring
                && this.weekdaySelection.isValid()
                || this.dateField.isValid()
            )
            && timebox.getComponent('hours').isValid()
            && timebox.getComponent('minutes').isValid()
            && timebox.getComponent('seconds').isValid()
        );
    },

    /**
     * Returns the currently set form values
     * Use isValid() to determine if current values are valid
     * @return {Object} Object of form:
     *      oneTimeEvent: {Boolean} Whether the date or days field is relevant. date on true,
     *      date: {Object} Ext.Date object
     *      days: {Array} Array of int, selected day index (mon=0..sun=6)
     *      hour: {Number} Hour component of time
     *      minute: {Number} Minute component of time
     */
    getValues: function() {
        var timebox = this.getComponent('time');
        var ret = {
            recurring: this.recurring,
            recurrence: this.recurring
                        ? this.weekdaySelection.getValues()
                        : [this.dateField.getValue()], // TODO: support multiple dates
            hours: timebox.getComponent('hours').getValue(),
            minutes: timebox.getComponent('minutes').getValue(),
            seconds: timebox.getComponent('seconds').getValue()
        };
        return ret;
    }
});

/**
 * DSS Activity Configuration Panel
 */
Ext.define('DSS.component.ActivityConfigurator', {
    alias: ['widget.dssActivityConfigurator'],
    extend: 'DSS.component.Base',
    mixins: {
        observable: 'Ext.util.Observable'
    },
    uses: [
        'DSS.apartment.ApartmentStore',
        'Ext.data.Store',
        'Ext.form.FieldContainer',
        'Ext.form.field.Checkbox',
        'Ext.form.field.ComboBox',
        'Ext.layout.container.Anchor',
        'Ext.layout.container.Column',
        'Ext.slider.Single',
        'Ext.Template'
    ],
    /**
     * Gets the apartment store
     * Be careful about what you do with it (esp. filtering)
     * @return {Object} The apartment store or null if uninitialized
     */
    getApartmentStore: function() {
        return this.apartmentStore;
    },
    /** @private */
    apartmentStore: null,
    /** @private */
    values: {
        targetType: null,
        zone: null,
        group: null,
        device: null,
        actionType: null,
        outputValue: null,
        scene: null,
        valid: false
    },
    /** @private */
    setValid: function(valid) {
        this.values.valid = valid;
        this.fireEvent('valuechanged', valid, this.values);
    },

    /** @private A component reset is in progress, discard certain events */
    resetting: false,

    /**
     * Reset all values to default (unset) or specified values
     * @param values (optional) values to reset to - default if unset
     */
    resetComponent: function(values) {
        var me = this;
        me.values = {
            targetType: null,
            zone: null,
            group: null,
            device: null,
            actionType: null,
            outputValue: null,
            scene: null,
            valid: false
        };

        // Reset to unset values
        // reset target box
        me.targetBox[0].setValue(undefined);
        var targetBox = me.targetBox[0].up();
        var i = targetBox.items.length;
        while(--i > 0) {
            targetBox.remove(targetBox.items.getAt(i), true);
            me.targetBox[i] = null;
        }
        // reset action box
        me.actionBox[0].setValue(undefined);
        me.actionBox[0].setDisabled(values === undefined);
        var actionBox = me.actionBox[0].up();
        i = actionBox.items.length;
        while(--i > 0) {
            actionBox.remove(actionBox.items.getAt(i), true);
            me.actionBox[i] = null;
        }

        if (values !== undefined) {
            // We fire select every time on a combo box, to kick off
            // the rendering process that is handled there.
            // This is not done when programatically select'ing
            me.resetting = true;

            // Reset to specific values
            me.targetBox[0].select(values.targetType);
            me.values.targetType = values.targetType;
            var targetType = me.targetTypeStore.getById(values.targetType);
            me.targetBox[0].fireEvent('select', me.targetBox[0], [targetType]);
            me.createZoneBox(targetBox, values.targetType, function() {
                // callback when the zone box is created and loaded
                me.targetBox[1].select(values.zoneId);
                me.values.zone = me.targetBox[1].store.getById(values.zoneId);
                me.targetBox[1].fireEvent('select', me.targetBox[1], [me.values.zone]);
                if (values.targetType == DSS.component.TargetType.ZONE) {
                    // create group (color) choice box
                    var groupStore = me.values.zone.groups();
                    me.createGroupBox(targetBox, groupStore);
                    me.targetBox[2].select(values.groupId);
                    me.values.group = me.targetBox[2].store.getById(values.groupId);
                    me.targetBox[2].fireEvent('select', me.targetBox[2], [me.values.group]);
                } else if (values.targetType == DSS.component.TargetType.DEVICE) {
                    // create device choice box
                    var deviceStore = me.values.zone.devices();
                    me.createDeviceBox(targetBox, deviceStore);
                    me.targetBox[2].select(values.deviceId);
                    me.values.device = me.targetBox[2].store.getById(values.deviceId);
                    me.targetBox[2].fireEvent('select', me.targetBox[2], [me.values.device]);
                }

                // Set the action boxes
                me.actionBox[0].select(values.actionType);
                me.values.actionType = values.actionType;
                var actionType = me.actionTypeStore.getById(values.actionType);
                me.actionBox[0].fireEvent('select', me.actionBox[0], [actionType]);
                if (values.actionType == DSS.component.ActionType.ACTIVITY) {
                    me.createSceneBox(actionBox, values.zoneId, function(success) {
                        if (success) {
                            var sceneStore = me.actionBox[1].store;
                            var scene = sceneStore.getById(values.sceneId);
                            me.values.scene = scene;
                            if (scene && sceneStore.indexOf(scene) === -1)
                                me.actionBox[2].setValue(true);
                            me.actionBox[1].select(values.sceneId);
                            me.actionBox[1].fireEvent('select', me.actionBox[1], [scene]);
                            me.values.valid = true;
                        }
                        me.resetting = false;
                    });
                } else if (values.actionType == DSS.component.ActionType.OUTPUT) {
                    me.createSetOutputBox(actionBox, false, values.outputValue);
                    me.values.outputValue = values.outputValue;
                    me.values.valid = true;
                    me.resetting = false;
                } else if (values.actionType == DSS.component.ActionType.BLINK) {
                    me.values.valid = true;
                    me.resetting = false;
                }
            }); // createZoneBox
        } // values !== undefined
    }, // resetComponent

    /** The target box containing the choice of the target type and subsequent choices */
    targetBox: [null, null, null],
    /** The action box containing the choice of the action type and subsequent choices */
    actionBox: [null, null],

    /** The store holding the choice of possible target types */
    targetTypeStore: null,
    /** The store holding the choice of possible action types */
    actionTypeStore: null,

    initComponent: function() {
        var me = this;

        this.addEvents({
            /**
             * @event valuechanged
             * @param valid {Boolean} True when the current values are valid
             * @param values {Object} Object containing the current values
             */
            valuechanged: true
        });

        Ext.define('TargetTypeModel', {
            extend: 'Ext.data.Model',
            idProperty: 'target',
            fields: ['target', 'icon', 'text']
        });
        me.targetTypeStore = Ext.create('Ext.data.Store', {
            model: 'TargetTypeModel',
            data: [
                { target: DSS.component.TargetType.ZONE, icon: null, text: _('Room') },
                { target: DSS.component.TargetType.DEVICE, icon: null, text: _('Device') }
            ]
        });
        Ext.define('ActionTypeModel', {
            extend: 'Ext.data.Model',
            idProperty: 'action',
            fields: ['action', 'icon', 'text']
        });
        me.actionTypeStore = Ext.create('Ext.data.Store', {
            model: 'ActionTypeModel',
            data: [
                { action: DSS.component.ActionType.ACTIVITY, icon: null, text: _('Call Activity') },
                { action: DSS.component.ActionType.OUTPUT, icon: null, text: _('Set output') },
                { action: DSS.component.ActionType.BLINK, icon: null, text: _('Blink') }
            ]
        });

        me.targetBox[0] = me.createComboBox(me.targetTypeStore, me.onTargetChanged, {
            fieldLabel: _('Target'),
            valueField: 'target'
        });
        me.actionBox[0] = me.createComboBox(me.actionTypeStore, me.onActionChanged, {
            fieldLabel: _('Action'),
            disabled: true,
            valueField: 'action'
        });
        Ext.apply(me, {
            layout: {
                type: 'column',
                width: 500
            },
            bodyPadding: '5',
            defaults: {
                xtype: 'container',
                columnWidth: 0.5,
                layout: {
                    type: 'anchor',
                    align: 'stretch',
                    anchor: '100%'
                }
            },
            items: [
                { items: [me.targetBox[0]] },
                { items: [me.actionBox[0]] }
            ],
        });

        this.callParent(arguments);
        this.apartmentStore = Ext.create('DSS.apartment.ApartmentStore');
        this.apartmentStore.load();
    },

    /**
     * Enables or disables the actionBox.
     * Clears associated values and related boxes when disabling
     * @param disabled {Boolean} True to disable, false to enable
     */
    disableActionBox: function(disabled) {
        var me = this;
        me.actionBox[0].setDisabled(disabled);
        var box = me.actionBox[0].up();
        var i = 1;
        while(box.items.length > 1) {
            box.remove(box.items.getAt(1))
            me.actionBox[i++] = null;
        }
        me.actionBox[0].setValue(undefined);

        // reset
        me.values.actionType = null;
        me.values.outputValue = null;
        me.values.scene = null;
        me.setValid(false);
    },

    /**
     * @private
     * Creates an Ext.form.Fieldcontainer containing an output selector
     * @param output {Number} Percentage to set on the output slider (Optional)
     * @return {Object} An Ext.form.Fieldcontainer
     */
    createSliderBox: function(output) {
        var out = output || 0;
        return Ext.create('Ext.form.FieldContainer', {
            fieldLabel: _('Output'),
            labelWidth: 45,
            items: [{
                xtype: 'slider',
                itemId: 'output',
                name: 'output',
                minValue: 0,
                maxValue: 100,
                value: out,
                width: 100
            }]
        });
    },

    /**
     * @private
     * Replace a box's last content while leaving the first few children
     * @param box {Object} The box
     * @param skip {Number} The number of items to leave
     * @param content {Object} The new content
     */
    setBox: function(box, skip, content) {
        while(skip < box.items.length) {
            box.remove(box.items.getAt(skip))
        }
        if (content)
            box.add(content);
    },

    /**
     * @private
     * Creates a new combobox with a few default settings
     * @param store {Object} The store to use
     * @param selectHandler {Function} The handler to bind to the select listener
     * @param config {Object} The config to apply to change defaults (Optional)
     * @return {Object} The created combobox
     */
    createComboBox: function(store, selectHandler, customConfig) {
        var me = this;
        var iconTpl = Ext.create('Ext.Template', [
                // The pics are 16x16, +5 padding = 21
                '<div style="',
                    '<tpl if="icon">',
                        'background:left center no-repeat url(\'images/dss/{icon}\');',
                    '</tpl>',
                    'min-height:16;',
                    'padding-left:21px;',
                '">{',
                (customConfig ? customConfig.displayField || 'text' : 'text'),
                '}</div>'
            ]
        );
        var config = {
            editable: false,
            forceSelection: true,
            fieldLabel: _('Room'),
            labelWidth: 45,
            listConfig: { itemTpl: iconTpl },
            store: store,
            // without lastQuery, the combobox applies its own filter on the
            // store, discarding existing filters.
            lastQuery: '',
            queryMode: 'local',
            valueField: 'id',
            listeners: {
                select: Ext.bind(selectHandler, me)
            },
            width: 225
        };
        if (customConfig)
            Ext.apply(config, customConfig);
        var box = Ext.create('Ext.form.field.ComboBox', config);
        box.on('select', function(field, value, options) {
            var bg = 'background:none;';
            var icon = (value.length > 0 ? value[0].get('icon') : null);
            if (icon) {
                var url = 'images/dss/' + icon;
                bg = "background:left center no-repeat url('"+ url +"');";
            }
            field.setFieldStyle(bg + 'padding-left:21px;');
        });
        return box;
    },

    /** @private Called when the event's target is changed */
    onTargetChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        var target = field.getValue(); // one of DSS.component.TargetType
        me.values.targetType = target;
        me.values.zone = null;
        me.values.group = null;
        me.values.device = null;

        me.createZoneBox(field.up(), target);

        // reset
        me.disableActionBox(true);
        me.targetBox[2] = null;
    },

    /** @private */
    createZoneBox: function(targetBox, targetType, callback) {
        var me = this;
        var createBox = function(targetBox, targetType, callback) {
            // can't create before the store is loaded
            // or .first() returns undefined
            var zoneStore = me.apartmentStore.first().zones();
            zoneStore.clearFilter(true);
            zoneStore.filter({ property: 'isPresent', value: true });
            var handler;
            if (targetType == DSS.component.TargetType.ZONE)
                handler = me.onTargetZoneChanged;
            else if (targetType == DSS.component.TargetType.DEVICE)
                handler = me.onDeviceZoneChanged;
            else
                handler = Ext.emptyFn;
            var zonebox = me.createComboBox(zoneStore, handler);
            me.setBox(targetBox, 1, zonebox);
            me.targetBox[1] = zonebox;
            if (callback && Ext.isFunction(callback))
                callback();
        };

        if (me.apartmentStore.isLoading()) {
            me.apartmentStore.load = Ext.bind(createBox, me, [targetBox, targetType, callback]);
        } else {
            createBox(targetBox, targetType, callback);
        }
    },

    /** @private Called when the event's zone (room) on a `zone' target is changed */
    onTargetZoneChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        me.createGroupBox(field.up(), value[0].groups());
        me.values.zone = value[0];

        // reset
        me.disableActionBox(true);
        me.values.group = null;
    },

    /** @private */
    createGroupBox: function(targetBox, groupStore) {
        var me = this;
        groupStore.clearFilter(true);
        groupStore.filterBy(function(item) {
            return (
                item.get('id') === 0
                || (item.get('isPresent')
                    && item.devices().getCount() > 0)
            );
        });
        var groupbox = me.createComboBox(groupStore, me.onTargetGroupChanged, {
            fieldLabel: _('Group'),
        });
        me.setBox(targetBox, 2, groupbox);
        me.targetBox[2] = groupbox;
    },

    /** @private Called when the event's group (color) on a `zone' target is changed */
    onTargetGroupChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        me.disableActionBox(false);
        me.values.group = value[0];
    },

    /** @private Called when the event's zone (room) on a `device' target is changed */
    onDeviceZoneChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        var deviceStore = value[0].devices();
        me.createDeviceBox(field.up(), deviceStore);
        me.values.zone = value[0];

        // reset
        me.values.device = null;
        me.disableActionBox(true);
    },

    /** @private */
    createDeviceBox: function(targetBox, deviceStore) {
        var me = this;
        var box = me.createComboBox(deviceStore, me.onDeviceChanged, {
            fieldLabel: _('Device'),
        });
        me.setBox(targetBox, 2, box);
        me.targetBox[2] = box;
    },

    /** @private Callend when a device is chosen in a given zone (room) */
    onDeviceChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        me.disableActionBox(false);

        // reset
        me.values.device = value[0];
    },

    /** @private Callend when the action type is changed */
    onActionChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        var action = field.getValue();
        me.values.actionType = action;

        if (action == DSS.component.ActionType.ACTIVITY) {
            // The action is to call an activity
            me.setValid(false);
            var zoneId = me.values.zone.get('id'); // Group (color) is ignored right now
            me.createSceneBox(field.up(), zoneId);

        } else if (action == DSS.component.ActionType.OUTPUT) {
            // The action is 'setOutput'
            me.setValid(false);
            me.createSetOutputBox(field.up(), true);

        } else if (action == DSS.component.ActionType.BLINK) {
            // The action is to blink
            me.values.outputValue = null;
            me.values.scene = null;
            me.setBox(field.up(), 1, null);
            me.setValid(true);

        } else {
            // shouldn't happen
            Ext.Msg.alert(_('Error'), _('Unknown action type') + '`' + action + '\'');
        }
    },

    /** @private */
    createSetOutputBox: function(targetBox, fireChange, value) {
        var me = this;
        var box = me.createSliderBox(value);
        var outputbox = box.getComponent('output');
        outputbox.on('change', function(slider, val, thumb, obj) {
            me.values.outputValue = val;
            me.setValid(true);
        });
        if (fireChange)
            outputbox.fireEvent('change', outputbox, 0, outputbox.thumbs[0], null);
        me.actionBox[1] = box;
        me.setBox(targetBox, 1, box);
    },

    /**
     * @private
     * @param targetBox {Object} Box to append the component to
     * @param zoneId {Number} Id to fetch reachable scenes for
     * @param callback {Function} Callback when scenes are fetched.
     *      The sigle argument passed is success {Boolean} which indicates
     *      whether the store was fetched successfully.
     */
    createSceneBox: function(targetBox, zoneId, callback) {
        var me = this;
        DSS.apartment.Helper.createReachableSceneStore(
            zoneId,
            function(success, reachableSceneStore, filterFn) {
                var box = null;
                if (success) {
                    scenebox = me.createComboBox(
                        reachableSceneStore,
                        me.onSceneChanged,
                        { fieldLabel: _('Activity') }
                    );
                    advbox = Ext.create('Ext.form.field.Checkbox', {
                        boxLabel: _('More activities'),
                        padding: '0 0 0 50', // 45px label width + 5px border
                        handler: function(field, checked) {
                            if (checked) {
                                reachableSceneStore.clearFilter(false);
                                // Hack to make the picker scrollable..
                                // Somehow it doesn't respect maxHeight
                                var picker = me.actionBox[1].getPicker();
                                picker.setHeight(picker.maxHeight);
                            } else {
                                reachableSceneStore.filterBy(filterFn);
                            }
                        }
                    });
                    me.setBox(targetBox, 1, scenebox);
                    me.setBox(targetBox, 2, advbox);
                    me.actionBox[2] = advbox;
                } else {
                    scenebox = me.createComboBox(undefined, Ext.emptyFn, {
                        activeError: _("Couldn't reachable scenes"),
                        disabled: true,
                        fieldLabel: _('Activity')
                    });
                    me.setBox(targetBox, 1, scenebox);
                }
                me.actionBox[1] = scenebox;
                if (callback && Ext.isFunction(callback))
                    callback(success);
            }
        );
    },

    /** @private Called when the scene is chosen for an action */
    onSceneChanged: function(field, value, options) {
        var me = this;
        if (me.resetting)
            return;
        me.values.scene = value[0];
        me.setValid(true);
    },

    /**
     * Returns the currently set form values
     * @return {Object} Currently set values
     */
    getValues: function() {
        return this.values;
    }
});

