/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

/**
 * An event panel to display timed events
 * @class DSS.addon.TimedEvent.EventPanel
 * @extends Ext.grid.Panel
 */
Ext.define('DSS.addon.TimedEvents.EventPanel', {

    extend: 'Ext.grid.Panel',
    uses: [
        'DSS.addon.TimedEventsEditor',
        'DSS.json.Event',
        'Ext.button.Button',
        'Ext.data.Model',
        'Ext.data.Store',
        'Ext.form.Panel',
        'Ext.grid.Column',
        'Ext.JSON',
        'Ext.selection.RowModel',
        'Ext.window.Window'
    ],

    config: {
        enableColumnHide: false,
        enableColumnMove: false,
        selType: 'rowmodel',
        viewConfig: { forceFit: true }
    },

    /** The event store */
    store: null,
    /** The window holding the timedEventsEditor */
    editorWindow: null,
    /** The timedEventsEditor inside editorWindow */
    timedEventsEditor: null,
    /** The currently selected event (null if none) */
    curSelectedEvent: null,
    /** DSS JSon class */
    property: 'DSS.json.Property',

    constructor: function(config) {
        this.initConfig(config);
        this.property = Ext.create(this.property, config);
        this.callParent(arguments);
    },

    initComponent: function() {
        var me = this;
        Ext.define('ActionModel', {
            extend: 'Ext.data.Model',

            fields: [
                { name: 'zone', type: 'integer', defaultValue: -1 },
                { name: 'group', type: 'integer', defaultValue: -1 },
                { name: 'scene', type: 'integer', defaultValue: -1 },
                { name: 'blink', type: 'boolean', defaultValue: false },
                'device',
                { name: 'outputValue', type: 'integer', defaultValue: -1 }
            ],

            getParams: function(reconstruct) {
                var ret = {};

                var tmp = this.get('zone');
                if (tmp !== -1)
                    ret.zone = tmp;

                tmp = this.get('group');
                if (tmp !== -1)
                    ret.group = tmp;

                tmp = this.get('scene');
                if (tmp !== -1)
                    ret.scene = tmp;

                tmp = this.get('blink');
                if (tmp !== false)
                    ret.blink = true;

                tmp = this.get('device');
                if (tmp)
                    ret.device = tmp;

                tmp = this.get('outputValue');
                if (tmp !== -1)
                    ret.outputValue = tmp;

                if (reconstruct)
                    ret = me.timedEventsEditor.reconstructValues(ret);

                return ret;
            }
        });
        Ext.define('TimedEventsModel', {
            extend: 'Ext.data.Model',
            statics: {
                create: function(params, handler) {
                    var model = new TimedEventsModel(params, params.name);
                    var event = Ext.create('DSS.json.Event', {name: 'timed-events'});
                    event.raise({
                            action_type: 'add',
                            event_name: params.name,
                            params: Ext.JSON.encode(model.getParams())
                        },
                        {
                            success: Ext.bind(handler, me, [model]),
                            failure: function() {
                                Ext.Msg.alert(_('Error'), _('Couldn\'t create timed event on server'));
                            }
                        }
                    );
                    return model;
                }
            },
            proxy: {
                type: 'ajax',
                url: '/json/property/query?query=/scripts/timed-events'
                     + '/events/*(name,disabled,time,recurring)'
                     + '/actions/*(blink,scene,outputValue,group,zone,device)',
                pageParam: undefined,
                limitParam: undefined,
                startParam: undefined,
                reader: {
                    type: 'json',
                    root: 'result.events',
                    successProperty: 'ok'
                }
            },
            recLoadCount: 0,
            recLoader: function(val) {
                if (this.data.recurrence) {
                    this.data.recurrence += (',' + val);
                } else {
                    this.data.recurrence = val;
                }
                if (--this.recLoadCount == 0) {
                    this.commit();
                }
            },
            constructor: function(data, id) {
                var eventpanel = me;
                var mi = this;
                var undef = (data.recurrence === undefined);
                var actions = undefined;

                if (!undef) {
                    // serialize dates
                    data.recurrence = data.recurrence.join(',');
                    actions = data.actions;
                    delete data.actions;
                }

                // Init the model and set default values
                mi.callParent(arguments);
                var recProp = 'events/'+ mi.getId() +'/recurrence';

                if (undef) {
                    eventpanel.property.getChildren(
                        recProp,
                        function(arr) {
                            if (data.recurring) {
                                var res = [];
                                Ext.Array.each(arr, function(el) {
                                    if (!isNaN(el.name))
                                        res.push(parseInt(el.name, 10));
                                });
                                mi.set('recurrence', res.join(','));
                                mi.commit();
                            } else {
                                mi.recLoadCount = arr.length;
                                // we need to fetch the values individually
                                Ext.Array.each(arr, function(el) {
                                    eventpanel.property.getString(
                                        recProp +'/'+ el.name,
                                        Ext.bind(mi.recLoader, mi)
                                    );
                                });
                            }
                        }
                    );
                } else {
                    // create actions
                    var store = mi.actions();
                    Ext.Array.each(actions, function(a) {
                        store.add(new ActionModel(a));
                    });
                }
            },
            idProperty: 'name',
            fields: [
                'name',
                { name: 'disabled', type: 'boolean', defaultValue: false },
                'time',
                { name: 'recurring', type: 'boolean', defaultValue: false },
                'recurrence'
            ],
            hasMany: [{ name: 'actions', model: 'ActionModel' }],
            getParams: function(reconstruct) {
                var actions = [];
                Ext.Array.each(this.actions().data.items, function(a) {
                    actions.push(a.getParams(reconstruct));
                });
                var recurring = this.get('recurring');
                var recurrence = [];
                if (recurring) {
                    Ext.Array.each(this.get('recurrence').split(','), function(r) {
                        recurrence.push(parseInt(r, 10));
                    });
                } else {
                    recurrence = this.get('recurrence').split(',');
                }
                return {
                    name: this.getId(),
                    actions: actions,
                    recurring: recurring,
                    recurrence: recurrence,
                    time: this.get('time')
                };
            },
            kill: function() {
                var me = this;
                var event = Ext.create('DSS.json.Event', {name: 'timed-events'});
                event.raise(
                    {
                        action_type: 'delete',
                        event_name: me.getId()
                    },
                    function() { me.store.remove(me); }
                );
            },
            disable: function(val) {
                var eventpanel = me;
                var mi = this;
                if (mi.get('disabled') == val)
                    return;
                eventpanel.property.setBoolean(
                    'events/' + this.getId() + '/disabled',
                    !!val,
                    function() {
                        mi.set('disabled', !!val);
                        mi.commit();
                        eventpanel.fireEvent(
                            'selectionchange',
                            eventpanel.getView(),
                            eventpanel.getSelectionModel().getSelection()
                        );
                    }
                );
                /*
                // Set the params by raising an event. Not used because at
                // the time of return we're not certain it's set
                var event = Ext.create('DSS.json.Event', {name: 'timed-events'});
                event.raise({
                        action_type: (val ? 'disable' : 'enable'),
                        event_name: me.getId()
                    }, function() { me.set('disabled', val); }
                );
                */
            }
        });
        me.store = Ext.create('Ext.data.Store', { model: 'TimedEventsModel' });

        me.timedEventsEditor = Ext.create('DSS.addon.TimedEventsEditor', me.config);

        me.editorWindow = Ext.create('Ext.window.Window', {
            closeAction: 'hide',
            title: me.appName,
            modal: true, // Exclusive input for the window
            height: 500,
            width: 700,
            layout: 'fit',
            items: me.timedEventsEditor
        });

        me.timedEventsEditor.on({
            eventsave: function(values) {
                me.editorWindow.disable();
                var actions = [];
                var a;
                Ext.Array.each(values.actions, function(a) {
                    var action = {};
                    switch(a.targetType) {
                        case DSS.component.TargetType.ZONE:
                            action.zone = a.zoneId;
                            action.group = a.groupId;
                            break;
                        case DSS.component.TargetType.DEVICE:
                            action.device = a.deviceId;
                            break;
                    }
                    switch(a.actionType) {
                        case DSS.component.ActionType.BLINK:
                            action.blink = true;
                            break;
                        case DSS.component.ActionType.OUTPUT:
                            // Convert outputValue from 0..100 -> 0..255
                            action.outputValue = Math.floor(a.outputValue * 2.55);
                            break;
                        case DSS.component.ActionType.ACTIVITY:
                            action.scene = a.sceneId;
                            break;
                    }
                    actions.push(action);
                });

                // slightly reformat values to correspond to the params format
                var params = {
                    name: values.name,
                    actions: actions,
                    recurring: values.recurrence.recurring
                };
                var pad = function(i) { return i < 10 ? '0' + i : i; }; 
                params.time = [pad(values.recurrence.hours),
                               pad(values.recurrence.minutes),
                               pad(values.recurrence.seconds)].join(':');
                if (params.recurring) {
                    params.recurrence = values.recurrence.recurrence;
                } else {
                    params.recurrence = [];
                    Ext.Array.each(values.recurrence.recurrence, function(d) {
                        params.recurrence.push(Ext.Date.format(d, 'Y-m-d'));
                    });
                }

                if (me.curSelectedEvent) {
                    TimedEventsModel.create(params, function(model) {
                        // Note: see below (else case)
                        me.store.remove(me.curSelectedEvent);
                        me.store.loadData([model], true);
                    });
                } else {
                    TimedEventsModel.create(params, function(model) {
                        // Note that the event is not certain to have been
                        // processed on the dSS.
                        // It might still be in the event queue.
                        me.store.loadData([model], true);
                    });
                    // reset the current selection
                    var sel = me.getSelectionModel().getSelection();
                    if (sel.length > 0)
                        me.curSelectedEvent = sel[0];
                }
                me.editorWindow.hide();
                me.editorWindow.enable();
                me.timedEventsEditor.resetComponent();
            },
            eventcancel: function() {
                me.editorWindow.hide();
                // reset the currently selected line (was remove to add a new entry)
                if (!me.curSelectedEvent) {
                    var sel = me.getSelectionModel().getSelection();
                    if (sel.length > 0)
                        me.curSelectedEvent = sel[0];
                }
                me.timedEventsEditor.resetComponent();
            }
        }); // timedEventsEditor.on()
        me.on({
            selectionchange: function(view, selections, options) {
                var me = this;
                if (selections.length > 0) {
                    me.curSelectedEvent = selections[0];
                    Ext.getCmp('btn-edit').setDisabled(false);
                    Ext.getCmp('btn-delete').setDisabled(false);
                    var btnDisable = Ext.getCmp('btn-disable');
                    btnDisable.setDisabled(false);
                    if (me.curSelectedEvent.get('disabled')) {
                        btnDisable.setText(_('Enable event'));
                        btnDisable.setIconCls('icon-start-action');
                    } else {
                        btnDisable.setText(_('Disable event'));
                        btnDisable.setIconCls('icon-stop-action');
                    }
                } else {
                    me.curSelectedEvent = null;
                    Ext.getCmp('btn-edit').setDisabled(true);
                    Ext.getCmp('btn-disable').setDisabled(true);
                    Ext.getCmp('btn-delete').setDisabled(true);
                }
            }
        });
        me.callParent(arguments);
        me.initPage();
    }, // initComponent

    initPage: function() {
        var me = this;

        // Utterly ugly hack to fully initialize the component
        me.editorWindow.show();
        me.editorWindow.hide();
        // end of utterly ugly hack

        Ext.getCmp('btn-add').handler = function() {
            me.curSelectedEvent = null;
            me.editorWindow.show();
        };
        Ext.getCmp('btn-edit').handler = function() {
            me.timedEventsEditor.resetComponent(me.curSelectedEvent.getParams(true));
            me.editorWindow.show();
        };
        Ext.getCmp('btn-disable').handler = function() {
            var st = !me.curSelectedEvent.get('disabled');
            me.curSelectedEvent.disable(st);
        };
        Ext.getCmp('btn-delete').handler = function() {
            me.curSelectedEvent.kill();
        };
        me.store.load(
            function(records, op, success) {
                // TODO: remove this function (leave load)
                // left because it's a good debug point ;)
            }
        );
    },

    viewConfig: {
        stripeRows: true
    },

    tbar: {
        items: [{
            text: _('New event'),
            id: 'btn-add',
            iconCls: 'icon-add-action'
            // handler: function() { /* added in initPage() */ }
        }, {
            text: _('Edit event'),
            id: 'btn-edit',
            iconCls: 'icon-edit-action',
            disabled: true
            // handler: function() { /* added in initPage() */ }
        }, {
            text: _('Disable event'),
            id: 'btn-disable',
            iconCls: 'icon-stop-action',
            disabled: true
            // handler: function() { /* added in initPage() */ }
        }, '->', {
            text: _('Delete event'),
            id: 'btn-delete',
            iconCls: 'icon-delete-action',
            disabled: true
            // handler: function() { /* added in initPage() */ }
        }]
    },

    columns: [{
        id: 'col-status',
        header: _('Status'),
        dataIndex: 'disabled',
        renderer: function(val, metadata) {
            if (val) {
                metadata.tdCls = 'icon-stop-action';
                // Ext.String.htmlEncode(_('disabled'));
            } else {
                metadata.tdCls = 'icon-start-action';
                // Ext.String.htmlEncode(_('enabled'));
            }
            metadata.tdCls += ' icon-center-no-repeat';
        },
        resizable: false,
        width: 50
    }, {
        id: 'col-name',
        header: _('Event name'),
        dataIndex: 'name',
        width: 200
    }, {
        id: 'col-actions',
        header: _('Action'),
        dataIndex: 'name', // alibi index
        maintainFlex: true,
        flex: 5,
        sortable: false,
        renderer: function(val, metadata, record, colIdx, store, view) {
            var actions = record.actions();
            var num = actions.getCount();
            var res = [];
            for (var i = 0; i < num; ++i) {
                var values = this.timedEventsEditor.reconstructValues(
                    actions.getAt(i).data
                );
                res.push(values.text);
            }
            return res.join('<br>');
        }
    }, {
        id: 'col-time',
        header: _('Time'),
        dataIndex: 'time',
        width: 80
    }, {
        id: 'col-recurrence',
        header: _('Recurrence'),
        dataIndex: 'recurrence',
        flex: 2,
        maintainFlex: true,
        sortable: false,
        renderer: function(te) {
            if (!te)
                return '';

            var recs = te.split(',');
            if (recs[0].indexOf('-') === -1) {
                // Based on the format, we aren't recurring
                var WEEK = [_('MO'),_('TU'),_('WE'),_('TH'),_('FR'),_('SA'),_('SU')];
                var d = [null, null, null, null, null, null, null];
                Ext.Array.each(recs, function(el) {
                    var i = parseInt(el, 10);
                    d[i] = Ext.String.htmlEncode(WEEK[i]);
                });
                for (var i = 0; i < 7; ++i) {
                    if (!d[i]) {
                        d[i] = '<span style="'
                            + 'text-decoration:line-through;color:#aaaaaa">'
                            + Ext.String.htmlEncode(WEEK[i])
                            + '</span>'
                    }
                }
                return d.join(', ');
            } else {
                var d = [];
                Ext.Array.each(recs, function(el) {
                    d.push(new Date(el).toLocaleDateString());
                });
                return d.join('<br>');
            }
        }
    }]
});

/**
 * Timed Events Editor Panel
 * The panel will always have at least one action
 * @class DSS.addon.TimedEventsEditor
 * @extends Ext.panel.Panel
 */
Ext.define('DSS.addon.TimedEventsEditor', {
    alias: ['widget.timedEventsEditor'],
    extend: 'Ext.panel.Panel',
    mixins: {
        observable: 'Ext.util.Observable'
    },
    requires: [
        'DSS.component.ActionType'
    ],
    uses: [
        'DSS.apartment.Helper',
        'DSS.component.ActivityConfigurator',
        'DSS.component.RecurrenceSelection',
        'Ext.button.Button',
        'Ext.data.Store',
        'Ext.form.FieldSet',
        'Ext.form.field.Text',
        'Ext.form.Panel',
        'Ext.grid.Panel',
        'Ext.layout.container.Border',
        'Ext.layout.container.Fit',
        'Ext.selection.RowModel'
    ],
    config: {
        appName: ''
    },
    /**
     * Gets the activity configurator component
     * that holds the apartment store
     * be careful of what you do with it, esp. filtering
     * @return {Object} The activity configurator or null if not initialized
     */
    getActivityConfigurator: function() {
        return this.activityConfigurator;
    },
    /** The activity configuration component */
    activityConfigurator: null,
    /** The action store for actions associated with this timed event */
    actionStore: null,
    /** The recurrence selection component */
    recurrenceSelection: null,
    /** Textfield holding the event name */
    eventNameBox: null,
    /** @private The currently selected record */
    curActionStoreRecord: null,
    constructor: function(config) {
        this.initConfig(config);
        this.mixins.observable.constructor.call(config);
        this.callParent(arguments);
    },
    reconstructValues: function(partialValues) {
        if (partialValues.valid !== undefined)
            return partialValues;

        var values = Ext.apply({}, partialValues);
        var zoneStore = this.getActivityConfigurator().getApartmentStore().first().zones();
        if (!values.targetType) {
            if (values.device !== undefined && values.device.length !== 0) {
                values.targetType = DSS.component.TargetType.DEVICE;
                var zone = null;
                var device = undefined;
                for (var i = 0; i < zoneStore.getCount(); ++i) {
                    zone = zoneStore.getAt(i);
                    device = zone.devices().getById(values.device);
                    // assume i=0 implies 'all zones'
                    if (device && i !== 0) {
                        // only abort search if the device is not in the 'all zones'
                        // zone, otherwise keep looking
                        break;
                    }
                }
                if (device !== undefined) {
                    values.device = device;
                    // if wanted, we could choose the zone this device is in
                    // instead of using the "all rooms" zone 0
                    values.zone = zone;
                } else {
                    values.zone = zoneStore.first();
                }
                values.valid = (values.device !== undefined);

            } else if (values.zone >= 0) {
                values.targetType = DSS.component.TargetType.ZONE;
                values.zone = zoneStore.getById(values.zone);
                values.valid = (values.zone !== undefined);
            }
            if (values.group >= 0) {
                if (values.zone !== undefined) {
                    var group = values.zone.groups().getById(values.group);
                    if (group !== undefined)
                        values.group = group;
                }
            }
        }
        if (!values.actionType) {
            if (values.blink) {
                values.actionType = DSS.component.ActionType.BLINK;
            } else if (values.outputValue >= 0) {
                values.actionType = DSS.component.ActionType.OUTPUT;
                values.outputValue = Math.round(values.outputValue / 2.55);
            } else if (values.scene >= 0) {
                values.actionType = DSS.component.ActionType.ACTIVITY;
                var sceneStore = DSS.apartment.Helper.getDefaultSceneStore();
                values.scene = sceneStore.getById(values.scene);
            } else {
                values.valid = false;
            }
        }
        if (values.text === undefined)
            values.text = this.getShortDesc(values);
        if (values.icon === undefined)
            values.icon = this.getIcon(values);
        values.incomplete = !values.valid;
        return values;
    },
    getIcon: function(values) {
        if (!values.valid)
            return 'missing.png';

        var icon;
        if (values.actionType == DSS.component.ActionType.ACTIVITY) {
            icon = values.scene.get('icon');
            if (icon && icon.length > 0)
                return icon;
        }
        val = (values.group !== -1 ? values.group : values.device);
        if (val) {
            icon = val.get('icon');
            if (icon && icon.length > 0)
                return icon;
        }
        return null;
    },
    getShortDesc: function(values) {
        if (!values.valid)
            return _('Incomplete action...');

        if (values.actionType == DSS.component.ActionType.OUTPUT) {
            if (values.device) {
                if (values.zone.get('id') === 0) {
                    // Don't display 'set Dev X in All rooms to N%'
                    return Ext.String.format(
                        '{0}: {1}%', // _('Set {0} to {1}%'),
                        values.device.get('text'),
                        values.outputValue
                    );
                }
                return Ext.String.format(
                    '{0}@{1}: {2}%', // _('Set {0} in {1} to {2}%'),
                    values.device.get('text'),
                    values.zone.get('text'),
                    values.outputValue
                );
            }
            if (values.group) {
                return Ext.String.format(
                    '{0}@{1}: {2}%', // _('Set {0} in {1} to {2}%'),
                    values.group.get('text'),
                    values.zone.get('text'),
                    values.outputValue
                );
            }
        }
        if (values.actionType == DSS.component.ActionType.ACTIVITY) {
            if (values.device) {
                if (values.zone.get('id') === 0) {
                    // Don't display 'D in All rooms on N%'
                    return Ext.String.format(
                        _('{0} on {1}'),
                        values.scene.get('text'),
                        values.device.get('text')
                    );
                }
                return Ext.String.format(
                    _('{0} on {1}@{2}'),
                    values.scene.get('text'),
                    values.device.get('text'),
                    values.zone.get('text')
                );
            }
            if (values.group) {
                return Ext.String.format(
                    _('{0} on {1}@{2}'),
                    values.scene.get('text'),
                    values.group.get('text'),
                    values.zone.get('text')
                );
            }
        }
        if (values.actionType == DSS.component.ActionType.BLINK) {
            if (values.device) {
                if (values.zone.get('id') === 0) {
                    // Don't display 'D in All rooms on N%'
                    return Ext.String.format(
                        _('Blink {0}'),
                        values.device.get('text')
                    );
                }
                return Ext.String.format(
                    _('Blink {0}@{1}'),
                    values.device.get('text'),
                    values.zone.get('text')
                );
            }
            if (values.group) {
                return Ext.String.format(
                    _('Blink {0}@{1}'),
                    values.group.get('text'),
                    values.zone.get('text')
                );
            }
        }

        return 'error';
    },
    /**
     * Reset all values to default (unset) or specified values
     * @param values (optional) values to reset to - default if unset
     */
    resetComponent: function(values) {
        var me = this;
        var ap = Ext.getCmp('actionPanel');

        me.curActionStoreRecord = null;
        me.activityConfigurator.resetComponent();
        // HACK: removeAll() doesn't seem to affect the actionPanel (GridPanel)
        // so remove them one by one
        while (me.actionStore.getCount() > 0) {
            me.actionStore.removeAt(0);
        }

        if (values === undefined) {
            var incomplete = { valid: false };
            me.actionStore.add({
                icon: me.getIcon(incomplete),
                text: me.getShortDesc(incomplete)
            });
            me.eventNameBox.setValue('');
            me.eventNameBox.clearInvalid();
            me.recurrenceSelection.resetComponent();
            Ext.getCmp('btn-add-action').setDisabled(true);

        } else {
            me.eventNameBox.setValue(values.name);
            var ts = values.time.split(':');
            var rec = [];
            if (!values.recurring) {
                Ext.Array.each(values.recurrence, function(r) {
                    rec.push(new Date(r));
                });
            }
            var recValues = {
                hours: ts[0],
                minutes: ts[1],
                seconds: ts[2],
                recurring: values.recurring,
                recurrence: (values.recurring ? values.recurrence : rec)
            };
            me.recurrenceSelection.resetComponent(recValues);
            me.actionStore.add(values.actions);
            Ext.getCmp('btn-add-action').setDisabled(false);
        }

        // setting curActionStoreRecord is part of select()
        // me.curActionStoreRecord = me.actionStore.getAt(0);
        ap.getSelectionModel().select(0);

        var disabled = !values || me.actionStore.getCount() < 2;
        Ext.getCmp('btn-remove-action').setDisabled(disabled);
    },
    initComponent: function() {
        var me = this;

        me.addEvents({
            /**
             * Fired upon cancel button press
             * @event eventcancel
             */
            eventcancel: true,
            /**
             * Fired upon save button press
             * @event eventsave
             * @param values {Object} The event parameters of valid events
             */
            eventsave: true
        });

        var incomplete = { valid: false };
        Ext.define('EditorActionModel', {
            extend: 'Ext.data.Model',
            fields: [
                'text',
                'icon',
                'targetType',
                { name: 'zoneId', type: 'integer' },
                { name: 'groupId', type: 'integer' },
                'deviceId',
                'actionType',
                { name: 'outputValue', type: 'integer' },
                { name: 'sceneId', type: 'integer' },
                { name: 'incomplete', type: 'boolean', defaultValue: true }
            ],
            constructor: function(data, id) {
                var newdata;
                if (data.valid !== undefined) {
                    // convert data
                    newdata = {
                        text: data.text,
                        icon: data.icon,
                        targetType: data.targetType,
                        zoneId: (data.zone ? data.zone.getId() : null),
                        groupId: (data.group ? data.group.getId() : null),
                        deviceId: (data.device ? data.device.getId() : null),
                        actionType: data.actionType,
                        outputValue: data.outputValue,
                        sceneId: (data.scene ? data.scene.getId() : null),
                        incomplete: data.incomplete
                    };
                } else {
                    newdata = data;
                }
                this.callParent([newdata, id]);
            }
        });
        me.actionStore = Ext.create('Ext.data.Store', {
            model: 'EditorActionModel',
            data: [{
                icon: me.getIcon(incomplete),
                text: me.getShortDesc(incomplete)
            }],
            /**
             * Gets the actions stored in this store
             * @return {Array} An array of Objects of the form:
             *      text: {String} Textual representation of the action
             *      icon: {String} Icon name or null
             *      targetType: {String} One of DSS.component.TargetType
             *      zoneId: {Number} The zone Id if target is ZONE, null otherwise
             *      groupId: {Number} The group Id
             *      deviceId: {Number} The device Id if target is DEVICE, null otherwise
             *      actionType: {String} One of DSS.component.ActionType
             *      outputValue: {Number} Output value 0..100
             *      sceneId: {Number} Scene to be called
             */
            getValues: function() {
                var actions = [];
                var i, n = this.getCount();
                for (i=0; i<n; ++i) {
                    actions.push(Ext.clone(this.getAt(i).data));
                }
                return actions;
            }
        });

        var actionPanel = Ext.create('Ext.grid.Panel', {
            region: 'west',
            id: 'actionPanel',
            title: _('Actions'),
            flex: 1,
            collapsible: true,
            split: true, // resizable
            viewConfig: {
                deferInitialRefresh: false // otherwise selection is not available at creation time
            },
            listeners: {
                beforeselect: function(view, node, sel, options) {
                    if (me.curActionStoreRecord
                        && me.curActionStoreRecord.get('incomplete')
                        && node != me.curActionStoreRecord) {

                        // Doesn't display correctly with a window on top:
                        // Ext.Msg.alert(_('Missing data'),
                        //      _('Please complete the current action first')
                        // );
                        // TODO: ask to discard data
                        return false;
                    }
                    return true;
                },
                selectionchange: function(selModel, records) {
                    if (records.length == 0 // happens on abort (deselect)
                        || me.curActionStoreRecord == records[0]) {
                        return;
                    }
                    me.curActionStoreRecord = records[0];
                    if (records[0].get('incomplete')) {
                        me.activityConfigurator.resetComponent();
                    } else {
                        var values = Ext.clone(records[0].data);
                        me.activityConfigurator.resetComponent(values);
                    }
                }
            },
            columns: [
                {
                    dataIndex: 'icon',
                    hideable: false,
                    renderer: function(val, meta) {
                        if (val && val.length > 0) {
                            meta.style = 'padding: 0;height: 16px;'+
                            'background: no-repeat url(\'images/dss/'+ val +'\')';
                        }
                        return '&nbsp;';
                    },
                    padding: 0,
                    resizable: false,
                    width: 16
                }, {
                    header: _('Activity'),
                    dataIndex: 'text',
                    hideable: false,
                    resizable: false,
                    flex: 1
                }
            ],
            store: me.actionStore,
            tbar: {
                items: [
                    {
                        id: 'btn-add-action',
                        text: _('New action'),
                        iconCls: 'icon-add-action',
                        disabled: true,
                        handler: function() {
                            me.actionStore.add({
                                icon: me.getIcon(incomplete),
                                text: me.getShortDesc(incomplete)
                            });
                            actionPanel.getSelectionModel().select(
                                me.actionStore.getCount() -1
                            );
                            Ext.getCmp('btn-remove-action').setDisabled(false);
                        }
                    }, '->', {
                        id: 'btn-remove-action',
                        text: _('Remove'),
                        iconCls: 'icon-delete-action',
                        disabled: true,
                        handler: function() {
                            if (me.actionStore.getCount() > 1) {
                                me.actionStore.remove(me.curActionStoreRecord);
                                me.curActionStoreRecord = null;
                                actionPanel.getSelectionModel().select(0);
                            }
                            if (me.actionStore.getCount() === 1) {
                                this.setDisabled(true);
                            }
                        }
                    }
                ]
            }
        });

        me.recurrenceSelection = Ext.create('DSS.component.RecurrenceSelection');
        me.activityConfigurator = Ext.create('DSS.component.ActivityConfigurator');
        me.eventNameBox = Ext.create('Ext.form.field.Text', {
            name: 'eventname',
            fieldLabel: _('Event name'),
            allowBlank: false,
            width: 350
        });

        Ext.apply(me, {
            items: [
                {
                    region: 'north',
                    bodyPadding: 5,
                    items: [{
                        xtype: 'fieldset',
                        title: _('Recurrence'),
                        items: [
                            me.eventNameBox,
                            me.recurrenceSelection
                        ]
                    }]
                },
                actionPanel, // region: west
                {
                    region: 'center',
                    xtype: 'form',
                    layout: 'fit',
                    flex: 3,
                    bodyPadding: 5,
                    items: [{
                        xtype: 'fieldset',
                        title: _('Event Configuration'),
                        items: [me.activityConfigurator]
                    }]
                }
            ], // items
            buttons: [
                {
                    text: _('Abort'),
                    handler: function() {
                        me.fireEvent('eventcancel');
                    }
                }, {
                    text: _('Save'),
                    handler: function() {
                        if (me.isValid()) {
                            me.fireEvent('eventsave', me.getValues());
                        }
                    }
                }
            ]
        }); // Ext.apply

        // Initialize
        this.callParent(arguments);

        me.curActionStoreRecord = me.actionStore.getAt(0);
        me.activityConfigurator.on('valuechanged', function(valid, values) {
            // update selection text (valid/invalid)
            // save valid values in store
            var rec = me.curActionStoreRecord || me.actionStore.getAt(0);
            if (valid) {
                rec.set('targetType', values.targetType);
                rec.set('zoneId', values.zone.get('id'));
                if (values.group)
                    rec.set('groupId', values.group.get('id'));
                if (values.device)
                    rec.set('deviceId', values.device.get('id'));
                rec.set('actionType', values.actionType);
                if (values.outputValue)
                    rec.set('outputValue', values.outputValue);
                if (values.scene)
                    rec.set('sceneId', values.scene.get('id'));
                rec.set('incomplete', false);

                Ext.getCmp('btn-add-action').setDisabled(false);
            } else {
                Ext.getCmp('btn-add-action').setDisabled(true);
            }
            rec.set('icon', me.getIcon(values));
            rec.set('text', me.getShortDesc(values));
            rec.commit();
        });
    },
    layout: {
        type: 'border',
        align: 'left'
    },
    isValid: function() {
        var me = this;
        return (
            me.eventNameBox.isValid()
            && me.recurrenceSelection.isValid()
            // There can be at most one invalid entry
            && (me.actionStore.getCount() > 1
                || me.actionStore.getCount() == 1
                && me.actionStore.getAt(0).get('incomplete') == false
            )
        );
    },

    /**
     * Returns the currently set form values
     * @return {Object} Object of the form:
     *      name: {String} Name given to the event
     *      recurrence: {Object} Object of the form:
     *          recurrence: {Array} Array of int (the weekdays on recurring) / Ext.Date (non recurring) objects
     *          hours: {Number} Hour component of time
     *          minutes: {Number} Minute component of time
     *          seconds: {Number} Seconds component of time
     *      actions: {Object} Object of the form:
     *          text: {String} Textual representation of the action
     *          icon: {String} Icon name or null
     *          targetType: {String} One of DSS.components.TargetType
     *          zoneId: {Number} The zone Id if target is ZONE, null otherwise
     *          groupId: {Number} The group Id
     *          deviceId: {Number} The device Id if target is DEVICE, null otherwise
     *          actionType: {String} One of DSS.componenent.ActionType
     *          outputValue: {Number} Output value 0..100
     *          sceneId: {Number} Scene to be called
     */
    getValues: function() {
        return {
            name: this.eventNameBox.getValue(),
            recurrence: this.recurrenceSelection.getValues(),
            actions: this.actionStore.getValues()
        }
    }
});

